Class: Prick::Build::ExeNode
Instance Attribute Summary collapse
-
#pipe ⇒ Object
readonly
Using a pipe instead of just executing the command shaves off some deciseconds spent starting up bash.
Attributes inherited from Node
#args, #kind, #parent, #path, #phase, #source
Instance Method Summary collapse
-
#initialize(parent, phase, path, args = nil) ⇒ ExeNode
constructor
A new instance of ExeNode.
- #inspect ⇒ Object
Methods inherited from Node
#dump, #lines, #name, #prefix_lines, #schema, #schema=, #source_lines, #to_s
Constructor Details
Instance Attribute Details
#pipe ⇒ Object (readonly)
Using a pipe instead of just executing the command shaves off some deciseconds spent starting up bash. It expects the process to read database/username from standard input
128 129 130 |
# File 'lib/builder/node.rb', line 128 def pipe @pipe end |
Instance Method Details
#inspect ⇒ Object
135 |
# File 'lib/builder/node.rb', line 135 def inspect() "#{path} #{(args || []).join(" ")}" end |